	;
	; 6502 diagnostic utility for Creativision
	;
	; Disassembled and repaired by Oscar Toledo G.
	; https://nanochess.org/
	;
	; Creation date: Aug/08/2024.
	; Revision date: Aug/10/2024. Reached $0900 bytes repaired. Full ROM disassembled.
	; Revision date: Aug/11/2024. Correcting a few bugs. Fully corrected.
	;

	; There are several EPROM dumps provided, but fortunately
	; I could conclude that _2009-01-12_cpu_4k.bin could be
	; taken as good, except for the bit 1.
	;
	; Using this and other two dumps I built an automatic
	; program that generated a semi-usable binary, I passed
	; that binary through the disassembler available at:
	; https://www.masswerk.at/6502/disassembler.html
	;
	; Then I started crafting the assembler code based
	; on the disassembly, doing a comparison again the "good"
	; dump, and based on my own programming experience.
	;
	; Unfortunately only 2K of the cpu_4k.bin file were
	; available (comprising half of the main program),
	; the following code and data was harder to
	; decode.
	;
	; There are things that we'll never know until finding
	; another original 6502 Test EPROM, for example, the
	; cursor positioning code for LB247, it is $fd, but it
	; could be $ff in the original EPROM.
	;

	CPU 6502

	ORG $A000

LA000:
	JSR LB247
	; BIT
	DB $FD,$00,$00,$22,$29,$34,$FD
	LDA #$C0
	STA $F8
	STA.W $0000
	CLV
	LDA #$40
	BIT $F8
	BNE LA01B
	JMP LB55D
LA01B:
	BMI LA020
	JMP LB55D
LA020:
	BVS LA025
	JMP LB55D
LA025:
	CLV
	LDA #$00
	BIT.W $0000
	BEQ LA030
	JMP LB55D
LA030:
	BMI LA035
	JMP LB55D
LA035:
	BVS LA03A
	JMP LB55D

LA03A:	JSR LB247
	; INC
	DB $FD,$00,$00,$29,$2E,$23,$00,$00
	; DEC
	DB $24,$25,$23,$FD
	LDA #$7F
	STA $F9
	STA.W $0002
	LDA #$FF
	STA $F8
	STA.W $0001
	INC $F8
	BEQ LA05E
	JMP LB55D
LA05E:
	BPL LA063
	JMP LB55D
LA063:
	LDA $F8
	BEQ LA06A
	JMP LB55D
LA06A:
	DEC $F8
	BMI LA071
	JMP LB55D
LA071:
	BNE LA076
	JMP LB55D
LA076:
	LDA $F8
	CMP #$FF
	BEQ LA07F
	JMP LB55D
LA07F:
	LDX #$01
	DEC $F8,X
	LDA $F9
	CMP #$7E
	BEQ LA08C
	JMP LB55D
LA08C:
	INC $F8,X
	LDA $F9
	CMP #$7F
	BEQ LA097
	JMP LB55D
LA097:
	INC $F8,X
	BMI LA09E
	JMP LB55D
LA09E:
	INC.W $0002
	BMI LA0A6
	JMP LB55D
LA0A6:
	LDA.W $0002
	CMP #$80
	BEQ LA0B0
	JMP LB55D
LA0B0:
	DEC.W $0002
	BPL LA0B8
	JMP LB55D
LA0B8:
	LDA.W $0002
	CMP #$7F
	BEQ LA0C2
	JMP LB55D
LA0C2:
	INC.W $0001,X
	BMI LA0CA
	JMP LB55D
LA0CA:
	LDA.W $0002
	CMP #$80
	BEQ LA0D4
	JMP LB55D
LA0D4:
	DEC.W $0001,X
	BPL LA0DC
	JMP LB55D
LA0DC:
	LDA.W $0002
	CMP #$7F
	BEQ LA0E6
	JMP LB55D
LA0E6:
	JSR LB247
	; JMP
	DB $FD,$00,$00,$2A,$2D,$30,$FD
	JSR LA0F3
LA0F3:
	CLC
	PLA
	ADC #$11
	STA.W $0001
	PLA
	ADC #$00
	STA.W $0002
	JMP ($0001)

LA103:
	JSR LB247
	; PLA
	DB $FD,$00,$00,$30,$2C,$21,$FD
	LDA #$80
	PHA
	LDA #$00
	PHA
	LDA #$FF
	PLA
	BEQ LA11B
	JMP LB55D
LA11B:
	BPL LA120
	JMP LB55D
LA120:
	PLA
	BMI LA126
	JMP LB55D
LA126:
	BNE LA12B
	JMP LB55D
LA12B:
	CMP #$80
	BEQ LA132
	JMP LB55D
LA132:
	JSR LB247
	; BNE BVC
	DB $FD,$00,$00,$22,$2E,$25,$00,$00,$22,$36,$23,$FD
	
	DEC $FA
	BNE LA148
	JMP LB327
LA148:
	JMP LB30E


	TIMES $B000-$ DB $FF

LB000:
	LDA #$05	; Execute 5 times the test.
	STA $FA
	CLD
	LDX #$00	
.1:	LDA LB01C,X	; VDP registers.
	STA $3001
	TXA		
	ORA #$80
	STA $3001
	INX
	TXA		
	EOR #$08
	BNE .1
	JMP LB044

LB01C:
	DB $00,$D0,$02,$FF,$00,$7F,$07,$1E

LB024:

	LDY #$04
	LDA #$00
	STA $3001
	LDA #$48
	STA $3001
LB030:	LDX #$FF
LB032:	LDA #$00
	STA $3000
	JSR LB041
	DEX
	BNE LB032
	DEY
	BNE LB030
	RTS

	; VDP delay.
LB041:
	PHA
	PLA
	RTS

LB044:
	LDX #$00
	LDA #$00
	STA $3001
	LDA #$40
	STA $3001
LB050:	LDA LB06F,X
	STA $3000
	JSR LB041
	INX
	CPX #$00
	BNE LB050

LB05E:	LDA LB16F,X
	STA $3000
	JSR LB041
	INX
	CPX #$D8
	BNE LB05E

	JMP LB30E

LB06F:	DB $00
	DB $00,$00,$00,$00,$00,$00,$00,$00	; $00 Space
	DB $20,$20,$20,$20,$20,$00,$20,$00	; $01 !
	DB $50,$50,$50,$00,$00,$00,$00,$00	; $02 "
	DB $50,$50,$F8,$50,$F8,$50,$50,$00	; $03 #
	DB $20,$78,$A0,$70,$28,$F0,$20,$00	; $04 $
	DB $C0,$C8,$10,$20,$40,$98,$18,$00	; $05 %
	DB $20,$50,$50,$60,$A8,$90,$68,$00	; $06 &
	DB $30,$30,$20,$40,$00,$00,$00,$00	; $07 '
	DB $10,$20,$40,$40,$40,$20,$10,$00	; $08 (
	DB $40,$20,$10,$10,$10,$20,$40,$00	; $09 )
	DB $50,$50,$50,$00,$00,$00,$00,$00	; $0A Asterisk (again quotes)
	DB $00,$20,$20,$F8,$20,$20,$00,$00	; $0B +
	DB $00,$00,$00,$30,$30,$20,$40,$00	; $0C ,
	DB $00,$00,$00,$F8,$00,$00,$00,$00	; $0D -
	DB $00,$00,$00,$00,$00,$30,$30,$00	; $0E .
	DB $00,$08,$10,$20,$40,$80,$00,$00	; $0F /
	DB $70,$88,$98,$A8,$C8,$88,$70,$00	; $10 0
	DB $20,$60,$20,$20,$20,$20,$70,$00	; $11 1
	DB $70,$88,$08,$30,$40,$80,$F8,$00	; $12 2
	DB $70,$88,$08,$30,$08,$88,$70,$00	; $13 3
	DB $10,$30,$50,$90,$F8,$10,$10,$00	; $14 4
	DB $F8,$80,$F0,$08,$08,$88,$70,$00	; $15 5
	DB $30,$40,$80,$F0,$88,$88,$70,$00	; $16 6
	DB $F8,$08,$10,$20,$40,$80,$80,$00	; $17 7
	DB $70,$88,$88,$70,$88,$88,$70,$00	; $18 8
	DB $70,$88,$88,$78,$10,$20,$C0,$00	; $19 9
	DB $00,$30,$30,$00,$30,$30,$00,$00	; $1A :
	DB $30,$30,$00,$30,$30,$20,$40,$00	; $1B ;
	DB $10,$20,$40,$80,$40,$20,$10,$00	; $1C <
	DB $00,$00,$F8,$00,$F8,$00,$00,$00	; $1D =
	DB $80,$40,$20,$10,$20,$40,$80,$00	; $1E >
	DB $70,$88,$08,$10,$20,$00,$20	; $1F ?
LB16F:
	DB $00
	DB $70,$88,$68,$A8,$A8,$E8,$38,$00	; $20 @
	DB $20,$50,$88,$88,$F8,$88,$88,$00	; $21 A
	DB $F0,$88,$88,$F0,$88,$88,$F0,$00	; $22 B
	DB $70,$88,$80,$80,$80,$88,$70,$00	; $23 C
	DB $F0,$88,$88,$88,$88,$88,$F0,$00	; $24 D
	DB $F8,$80,$80,$F8,$80,$80,$F8,$00	; $25 E
	DB $F8,$80,$80,$F8,$80,$80,$80,$00	; $26 F
	DB $70,$88,$80,$B8,$88,$98,$68,$00	; $27 G
	DB $88,$88,$88,$F8,$88,$88,$88,$00	; $28 H
	DB $70,$20,$20,$20,$20,$20,$70,$00	; $29 I
	DB $08,$08,$08,$08,$88,$48,$30,$00	; $2A J
	DB $88,$90,$A0,$C0,$A0,$90,$88,$00	; $2B K
	DB $80,$80,$80,$80,$80,$80,$F8,$00	; $2C L
	DB $88,$D8,$A8,$A8,$88,$88,$88,$00	; $2D M
	DB $88,$88,$C8,$A8,$98,$88,$88,$00	; $2E N
	DB $70,$88,$88,$88,$88,$88,$70,$00	; $2F O
	DB $F8,$88,$88,$F8,$80,$80,$80,$00	; $30 P
	DB $70,$88,$88,$88,$A8,$90,$68,$00	; $31 Q
	DB $F0,$88,$88,$F0,$A0,$90,$88,$00	; $32 R
	DB $70,$88,$80,$70,$08,$88,$70,$00	; $33 S
	DB $F8,$20,$20,$20,$20,$20,$20,$00	; $34 T
	DB $88,$88,$88,$88,$88,$88,$70,$00	; $35 U
	DB $88,$88,$88,$88,$88,$50,$20,$00	; $36 V
	DB $88,$88,$88,$A8,$A8,$A8,$50,$00	; $37 W
	DB $88,$88,$50,$20,$50,$88,$88,$00	; $38 X
	DB $88,$50,$20,$20,$20,$20,$20,$00	; $39 Y
	DB $F8,$08,$10,$20,$40,$80,$F8	; $3A Z
	
LB247:
	CLC
	PLA
	STA $F0
	PLA
	STA $F1
	LDA $F0
	ADC #$01
	BCC LB257
	INC $F1
	CLC
LB257:	STA $F0
	LDY #$00
LB25B:	LDA ($F0),Y
	CMP #$FD
	CLC
	BNE LB265
	JMP LB29A

LB265:	STY $F2		; Screen pointer
	STY $F3
	JSR LB2C5
	STA $F4
	CMP #$00
	BEQ LB282
	CLC
	LDA $F2
LB275:	ADC #$28	; Next row (40 columns)
	BCC LB27C
	INC $F3		; Increase high-byte pointer
	CLC
LB27C:	DEC $F4		; Decrease count of rows.
	BNE LB275
	STA $F2
LB282:
	CLC
	INY
	LDA ($F0),Y
	JSR LB2C5
	ADC $F2
	BCC LB290
	INC $F3
	CLC
LB290:	STA $3001
	LDA $F3
	ADC #$48	; Screen located at $0800 in VRAM
	STA $3001
LB29A:	INY
	LDA ($F0),Y	; Read letter.
	CMP #$EC
	BEQ LB2DF
	CMP #$DD
	BEQ LB2E7
LB2A5:
	CMP #$FD
	BEQ LB2B5
	STA $3000	; Put in VRAM.
	JSR LB041
	INY
	LDA ($F0),Y
	JMP LB2A5
LB2B5:
	TYA		; Get current counter
	CLC
	ADC $F0		; Add to base address.
	BCC LB2BE
	INC $F1
	CLC
LB2BE:	TAX
	LDA $F1		; Put back in stack.
	PHA
	TXA
	PHA
	RTS		; Return to caller.

LB2C5:	STA $F5
	AND #$F0
	BNE LB2CE
	LDA $F5
	RTS

LB2CE:
	LSR A
	LSR A
	LSR A
	LSR A
	TAX
	LDA $F5
	AND #$0F
	CLC
LB2D8:
	ADC #$0a
	DEX
	BNE LB2D8
	CLC
	RTS

LB2DF:
	LDA $F4
	STA $3000
	JMP LB2B5

LB2E7:
	LDY #$00
	LDA $F4
	LSR A
	LSR A
	LSR A
	LSR A
	CLC
LB2F0:	CMP #$08
	BCC LB2FA
	CLC
	ADC #$15
	JMP LB2FC

LB2FA:	ADC #$10
LB2FC:	STA $3000
	DEY
	BNE LB30B
	LDY #$FD
	LDA $F4
	AND #$0F
	JMP LB2F0

LB30B:	JMP LB2B5

LB30E:	JSR LB024
	JSR LB247
	; 6502
	DB $00,$14,$16,$15,$10,$12,$00
	; TEST
	DB $34,$25,$33,$34
	DB $FD

	LDA #$00
	STA $F7
	JMP LB374

LB327:	LDA $F7
	CMP #$00
	BNE LB346
	JSR LB247
	; PASSED
	DB $21,$11,$30,$21,$33,$33,$25,$24,$00
	; ALL TESTS
	DB $21,$2C,$2C,$00,$34,$25,$33,$34,$33,$FD

	JMP LB327

LB346:	JSR LB247
	; INTERMITTENT
	DB $21,$14,$29,$2E,$34,$25,$32,$2D,$29,$34,$34,$25,$2E,$34,$FD
	JMP LB327

LB35B:	LDA #$19
	STA $3001
LB360:	LDA #$87
	STA $3001
	RTS

	; FAIL
LB366:	JSR LB247
	DB $FD,$26,$21,$29,$2C,$FD
	JSR LB35B	; Turn background red.
	LDA #$FF	; 
LB374:	STA $F7
	JSR LB247	
	; 1)
	DB $04,$01,$11,$09,$00
	; INTERNAL
	DB $29,$2e,$34,$25,$32,$2e,$21,$2C,$00
	; REGISTER
	DB $32,$25,$27,$29,$33,$34,$25,$32,$00
	; FUNCTIONAL
	DB $26,$35,$2e,$23,$34,$29,$2f,$2e,$21,$2C,$00
	DB $FD
	LDX #$00
	CPX #$00
	BNE LB366
	TXS
	TSX
	CPX #$00
	BNE LB366
	LDX #$FD
	CPX #$FD
	BNE LB366
	TXS
	TSX
	CPX #$FD
	BNE LB366
	LDY #$00
	CPY #$00
	BNE LB366
	LDY #$FF
	CPY #$FF
	BNE LB366
	LDA #$00
	CMP #$00
	BNE LB366
	PHA
	PLP
	PHP
	PLA
	AND #$CF
	CMP #$00
	BNE LB366
	LDA #$FF
	CMP #$FF
	BNE LB366
	PHA
	PLP
	PHP
	PLA
	CMP #$FF
	BEQ LB3E5
	LDA #$00
	PHA
	PLP
	JMP LB366
LB3E5:
	LDA #$00
	PHA
	PLP
	JSR LB247
	; GOOD
	DB $04,$33,$27,$2F,$2F,$24,$FD	; GOOD
	JMP LB406

LB3F6:
	JSR LB247
	; FAIL
	DB $FD,$26,$21,$29,$2C,$FD

LB3FF:	JSR LB35B
	LDA #$FF
	STA $F7
LB406:	JSR LB247
	; INTERNAL
	DB $06,$01,$12,$09,$00,$29,$2e,$34,$25,$32,$2e,$21,$2C,$00
	; REGISTER
	DB $32,$25,$27,$29,$33,$34,$25,$32,$00
	; SPEED TEST
	DB $33,$30,$25,$25,$24,$00,$34,$25,$33,$34,$00,$FD
	LDY #$00
	LDX #$00
LB430:	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INX
	INY
	TYA
	CLC
	ROL A
	ROL A
	ROL A
	ROL A
	STA $F6
	CPX $F6
	BNE LB3F6
	CPY #$10
	BNE LB430
	LDY #$00
	LDX #$00
LB455:	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INY
	INX
	TXA
	CLC
	ROL A
	ROL A
	ROL A
	ROL A
	STA $F6
	CPY $F6
	BNE LB3F6
	CPX #$10
	BNE LB455
	LDX #$10
LB478:	LDA #$AA
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	EOR #$FF
	CMP #$AA
	BEQ LB4A1
	JMP LB3F6

LB4A1:	DEX
	BNE LB478
	LDX #$FF
	TXS
	LDY #$0F
LB4A9:
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	PHA
	TSX
	DEY
	TYA
	ROL A
	ROL A
	ROL A
	ROL A
	ORA #$0F
	STA $F6
	CPX $F6
	BEQ LB4CB
	JMP LB3F6
LB4CB:
	CPY #$00
	BNE LB4A9
	LDX #$00
	TXS
	LDY #$00
LB4D4:
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	PLA
	INY
	TYA
	CLC
	ROL A
	ROL A
	ROL A
	ROL A
	STA $F6
	TSX
	CPX $F6
	BEQ LB4F5
	JMP LB3F6
LB4F5:	CPY #$10
	BNE LB4D4
	LDX #$FF
	TXS
	CLC
	CLI
	CLD
	LDX #$FF
LB501:
	SEC
	CLC
	BCC LB508
	JMP LB3F6
LB508:
	PHP
	PLA
	AND #$0D
	BEQ LB511
	JMP LB3F6
LB511:
	DEX
	BNE LB501
	LDX #$FF
	TXA
	EOR #$FF
LB519:	EOR #$FF
	EOR #$FF
	BEQ LB522
	JMP LB3F6
LB522:
	PHA
	ASL A
	PLA
	AND #$0F
	BEQ LB52C
	JMP LB3F6
LB52C:
	ROR A
	DEX
	BNE LB519
	LDX #$FF
LB532:	SEI
	CLI
	PHP
	PLA
	AND #$0D
	BEQ LB53D
	JMP LB3F6
LB53D:	DEX
	BNE LB532
	LDX #$FF
LB542:	SED
	CLD
	PHP
	PLA
	AND #$0D
	BEQ LB54D
	JMP LB3F6
LB54D:	DEX
	BNE LB542
	JSR LB247
	; GOOD
	DB $04,$33,$27,$2F,$2F,$24,$FD
	JMP LB5A3

LB55D:	JSR LB247
	; PROBABLY
	DB $20,$08,$30,$32,$2F,$22,$21,$22,$2C,$39,$00
	; HAVE
	DB $28,$21,$36,$25,$00
	; PROBLEM
	DB $30,$32,$2F,$22,$2C,$25,$2D,$00
	; ON
	DB $2F,$2E,$FD
	JSR LB247
	; LAST
	DB $21,$07,$2C,$21,$33,$34,$00
	; FEW
	DB $26,$25,$37,$00
	; INSTRVCTIONS
	DB $29,$2E,$33,$34,$32,$36,$23,$34,$29,$2F,$2E,$33,$00
	; SHOWN
	DB $33,$28,$2F,$37,$2E,$FD
	JSR LB35B	; Red background
	LDA #$FF	; 
	STA $F7
LB5A3:	JSR LB247
	; 3)
	DB $08,$01,$13,$09,$00
	; INSTRUCTION
	DB $29,$2E,$33,$34,$32,$35,$23,$34,$29,$2F,$2E,$00
	; DECODER
	DB $24,$25,$23,$2F,$24,$25,$32,$00
	; TEST
	DB $34,$25,$33,$34,$00,$FD
	JSR LB247
	; INSTRUCTION
	DB $10,$04,$29,$2E,$33,$34,$32,$35,$23,$34,$29,$2F,$2E,$00
	; TESTED/BEING TESTED:
	DB $34,$25,$33,$34,$25,$24,$0F,$22,$25,$29,$2E,$27,$00
	DB $34,$25,$33,$34,$25,$24,$1A,$FD
	JSR LB247
	; PHA  PHP
	DB $11,$00,$30,$28,$21,$00,$00,$30,$28,$30,$00,$00
	; PLP  JSR
	DB $30,$2C,$30,$00,$00,$2A,$33,$32,$00,$00
	; RTS
	DB $32,$34,$33,$FD
	JSR LB247
	; LDA
	DB $FD,$00,$00,$2C,$24,$21,$FD
	LDA #$00
	BEQ LB619
	JMP LB55D
LB619:
	LDA #$AA
	BMI LB620
	JMP LB55D

LB620:	BNE LB625
	JMP LB55D

LB625:	CMP #$AA
	BEQ LB62C
	JMP LB55D

LB62C:	LDA #$55
	BPL LB633
	JMP LB55D

LB633:	CMP #$55
	BEQ LB63A
	JMP LB55D

LB63A:	JMP LB641

LB63D:	DB $AA,$55,$55,$AA

LB641:	LDA LB63D
	CMP #$AA
	BEQ LB64B
	JMP LB55D

LB64B:	LDA LB63D+2
	CMP #$55
	BEQ LB655
	JMP LB55D

LB655:	LDX #$01
	LDA LB63D,X
	CMP #$55
	BEQ LB661
	JMP LB55D

LB661:	LDA LB63D+2,X
	CMP #$AA
	BEQ LB66B
	JMP LB55D

LB66B:	LDY #$01
	LDA LB63D,Y
	CMP #$55
	BEQ LB677
	JMP LB55D

LB677:	LDA LB63D+2,Y
	CMP #$AA
	BEQ LB681
	JMP LB55D

LB681:	LDA $F7
	CMP #$00
	BEQ LB68D
	CMP #$FF
	BEQ LB68D
	CMP #$00
LB68D:	LDA #$FA
	STA $F8
	LDA #$BF
	STA $F9
	LDA ($F8),Y
	CMP #$B0	; Definitive proof that $bffb should contain $B0
	BEQ LB69E
	JMP LB55D

LB69E:	LDX #$00
	LDA ($F8,X)
	CMP #$00
	BEQ LB6A9
	JMP LB55D

LB6A9:	LDA #$FF
	LDX #$01
	LDA ($F7,X)
	CMP #$00
	BEQ LB6B6
	JMP LB55D

LB6B6:	JSR LB247
	; BEQ
	DB $FD,$00,$00,$22,$25,$31,$00,$00
	; BMI BPL
	DB $22,$2D,$29,$00,$00,$22,$30,$2C,$FD
	JSR LB247
	; STA
	DB $FD,$00,$00,$33,$34,$21,$FD
	LDA #$AA
	STA $F8
	LDA #$55
	STA $F9
	LDA $F8
	CMP #$AA
	BEQ LB6E5
	JMP LB55D

LB6E5:	LDA $F9
	CMP #$55
	BEQ LB6EE
	JMP LB55D

LB6EE:
	LDA $F8
	STA.W $0001
	LDA $F9
	STA.W $0000
	LDA.W $0001
	CMP #$AA
	BEQ LB702
	JMP LB55D

LB702:	LDA.W $0000
	CMP #$55
	BEQ LB70C
	JMP LB55D

LB70C:	LDX #$01
	STA $F7,X
	LDA #$AA
	STA $F8,X
	LDA $F8
	CMP #$55
	BEQ LB71D
	JMP LB55D

LB71D:	LDA $F9
	CMP #$AA
	BEQ LB726
	JMP LB55D

LB726:	STA.W $0001,X
	LDA $F8
	STA.W $0000,X
	LDA.W $0001
	CMP #$55
	BEQ LB738
	JMP LB55D

LB738:	LDA.W $0002
	CMP #$AA
	BEQ LB742
	JMP LB55D

LB742:	LDY #$01
	STA.W $0000,Y
	LDA #$55
	STA.W $0001,Y
	LDA.W $0001
	CMP #$AA
	BEQ LB756
	JMP LB55D

LB756:	LDA.W $0000
	CMP #$55
	BEQ LB760
	JMP LB55D

LB760:	LDA #$01
	STA $F8
	LDA #$00
	STA $F9
	LDX #$01
	LDA #$55
	STA ($F7,X)
	LDA #$00
	STA $F8
	LDX #$02
	LDA #$AA
	STA ($F6,X)
	LDA.W $0001
	CMP #$55
	BEQ LB782
	JMP LB55D

LB782:	LDA.W $0000
	CMP #$AA
	BEQ LB78C
	JMP LB55D

LB78C:	LDA #$00
	STA $F8
	LDY #$01
	LDA #$AA
	STA ($F8),Y
	LDY #$00
	LDA #$55
	STA ($F8),Y
	LDA.W $0001
	CMP #$AA
	BEQ LB7A6
	JMP LB55D

LB7A6:	LDA.W $0000
	CMP #$55
	BEQ LB7B0
	JMP LB55D

LB7B0:	JSR LB247
	; ADC
	DB $FD,$00,$00,$21,$24,$23,$FD

LB7BA:	CLD
	CLC
	CLV
	LDA #$78
	ADC #$78
	BVS LB7C6
	JMP LB55D

LB7C6:	CLV
	BMI LB7CC
	JMP LB55D

LB7CC:	CMP #$F0
	BEQ LB7D3
	JMP LB55D

LB7D3:	CLC
	ADC #$10
	BCS LB7DB
	JMP LB55D

LB7DB:	ADC #$10
	BCC LB7E2
	JMP LB55D

LB7E2:	CMP #$11
	BEQ LB7E9
	JMP LB55D

LB7E9:	LDA #$66
	STA $F8
	STA.W $0001
	LDA #$99
	STA $F9
	STA.W $0000
	LDA #$AA
	; A = $aa
	CLC
	ADC $F8
	CMP #$10
	BEQ LB803
	JMP LB55D

	; Start of unknown territory.
LB803:
	ADC $F9
	CMP #$AA
	BEQ LB80C
	JMP LB55D

	; A = $aa
LB80C:
	CLC
	ADC.W $0001
	CMP #$10
	BEQ LB817
	JMP LB55D
LB817:
	ADC.W $0000
	CMP #$AA
	BEQ LB821
	JMP LB55D

	; A = $aa
LB821:
	LDX #$01
	CLC
	ADC $F7,X
	CMP #$10
	BEQ LB82D
	JMP LB55D

LB82D:	INX
	ADC $F7,X
	CMP #$AA
	BEQ LB837
	JMP LB55D

	; A = $aa
LB837:	LDY #$01
	LDX #$01
	CLC
	ADC.W $0000,X
	CMP #$10
	BEQ LB846
	JMP LB55D
LB846:
	DEX
	ADC.W $0000,X
	CMP #$AA
	BCS LB851
	JMP LB55D
	; A = $aa
LB851:
	CLC
	ADC.W $0000,Y
	CMP #$10
	BEQ LB85C
	JMP LB55D
LB85C:
	DEY
	ADC.W $0000,Y
	CMP #$AA
	BEQ LB867
	JMP LB55D

	; A = $aa
LB867:	PHA
	LDA #$00
	STA $F8
	LDA #$00
	STA $F9
	PLA
	INY
	CLC
	ADC ($F8),Y
	CMP #$10
	BEQ LB87C
	JMP LB55D
LB87C:
	DEY
	ADC ($F8),Y
	CMP #$AA
	BEQ LB886
	JMP LB55D

	; A = $aa
LB886:	INC $F8		; Addr=$0001
	INX		; X=$01
	CLC
	ADC ($F7,X)
	CMP #$10
	BEQ LB893
	JMP LB55D
LB893:	
	DEC $F8		; Addr=$0000
	ADC ($F7,X)
	CMP #$AA
	BEQ LB89E
	JMP LB55D
	; A = $aa
LB89E:
	SED
	CLC
	SBC #$34
	ADC #$34
	BCS LB8A9
	JMP LB55D
LB8A9:
	CMP #$10
	BEQ LB8B0
	JMP LB55D
LB8B0:
	CLD
	JSR LB247
	; BCC
	DB $FD,$00,$00,$22,$23,$23,$00,$00
	; BCS
	DB $22,$23,$33,$00,$00,$22,$36,$33
	; BVS
	DB $FD

	JSR LB247
	; CLC
	DB $FD,$00,$00,$23,$2C,$23,$00,$00
	; CLD CLI
	DB $23,$2C,$24,$00,$00,$23,$2C,$29
	; CLV
	DB $00,$00,$23,$2C,$36,$00,$00
	; SEC SED
	DB $33,$25,$23,$00,$00,$33,$25,$24
	; SEI
	DB $00,$00,$33,$25,$29,$FD

	CLC
	CLI
	CLD
	PHP
	PLA
	AND #$0F
	BEQ LB8F9
	JMP LB55D

LB8F9:
	SEC
	SED
	SEI
	LDA #$30
	EOR #$30
	PHP
	PLA
	AND #$0F
	CMP #$0F
	BEQ LB90B
	JMP LB55D

LB90B:	
	CLC
	CLI
	CLV
	CLD
	PHP
	PLA
	AND #$4D
	BEQ LB918
	JMP LB55D

LB918:
	SEI
	JSR LB247
	; TAX
	DB $FD,$00,$00,$34,$21,$38,$00,$00
	; TAY
	DB $34,$21,$39,$00,$00
	; TSX
	DB $34,$33,$38,$00,$00
	; TXA
	DB $34,$38,$21,$00,$00
	; TXS
	DB $34,$38,$33,$00,$00
	; TYA
	DB $34,$39,$21,$FD
	LDA #$00
	TAY
	BEQ LB944
	JMP LB55D
LB944:
	BVC LB949
	JMP LB55D
LB949:
	LDX #$00
	BEQ LB950
	JMP LB55D
LB950:
	LDA #$FF
	TAX
	BNE LB958
	JMP LB55D
LB958:
	BMI LB95D
	JMP LB55D
LB95D:
	CPX #$FF
	BEQ LB964
	JMP LB55D
	; Y = $00
	; X = $FF
LB964:
	LDA #$00
	TXA
	BNE LB96C
	JMP LB55D
LB96C:
	BMI LB971
	JMP LB55D
LB971:	
	CMP #$FF
	BEQ LB978
	JMP LB55D
LB978:
	LDX #$00
	TXA
	BEQ LB980
	JMP LB55D

LB980:
	BPL LB985
	JMP LB55D
LB985:
	CMP #$00
	BEQ LB98C
	JMP LB55D
LB98C:
	TAY
	BEQ LB992
	JMP LB55D
LB992:
	BPL LB997
	JMP LB55D
LB997:
	CPY #$00
	BEQ LB99E
	JMP LB55D
LB99E:
	LDA #$FF
	TAY
	BMI LB9A6
	JMP LB55D
	
LB9A6:	BNE LB9AB
	JMP LB55D

LB9AB:	CPY #$FF
	BEQ LB9B2
	JMP LB55D

LB9B2:	LDA #$00
	TYA
	BMI LB9BA
	JMP LB55D

LB9BA:	BNE LB9BF
	JMP LB55D

LB9BF:	CMP #$FF
	BEQ LB9C6
	JMP LB55D

LB9C6:	LDY #$00
	TYA
	BEQ LB9CE
	JMP LB55D

LB9CE:	BPL LB9D3
	JMP LB55D

LB9D3:	CMP #$00
	BEQ LB9DA
	JMP LB55D

LB9DA:	LDX #$55
	TXA
	STA.W $0156
	EOR #$FF
	STA.W $01AB	; @@@
	TXS
	PLA
	CMP #$55
	BEQ LB9EE
	JMP LB55D

LB9EE:	LDX #$AA
	TXS
	PLA
	CMP #$AA
	BEQ LB9F9
	JMP LB55D

LB9F9:	LDX #$55
	TXS
	LDY #$AA
	TSX
	CPX #$55
	BEQ LBA06
	JMP LB55D

LBA06:	LDX #$AA
	TXS
	LDY #$55
	TSX
	CPX #$AA
	BEQ LBA13
	JMP LB55D

LBA13:
	JSR LB247
	; LDX
	DB $FD,$00,$00,$2C,$24,$38,$00,$00
	; LDY
	DB $2C,$24,$39,$FD

	LDA #$55
	STA $F8
	STA.W $0001
	LDA #$AA
	STA $F9
	STA.W $0000
	LDX $F8
	BPL LBA37
	JMP LB55D

LBA37:
	LDY $F9
	BMI LBA3E
	JMP LB55D

LBA3E:
	CPX #$55
	BEQ LBA45
	JMP LB55D
 
LBA45:
	CPY #$AA
	BEQ LBA4C
	JMP LB55D

LBA4C:
	LDX $F9
	BMI LBA53
	JMP LB55D

LBA53:
	LDY $F8
	BPL LBA5A
	JMP LB55D

LBA5A:	CPX #$AA
	BEQ LBA61
	JMP LB55D

LBA61:	CPY #$55
	BEQ LBA68
	JMP LB55D

LBA68:	LDX.W $0001
	LDY.W $0000
	CPX #$55
	BEQ LBA75
	JMP LB55D

LBA75:
	CPY #$AA
	BEQ LBA7C
	JMP LB55D

LBA7C:
	LDY.W $0001
	LDX.W $0000
	CPX #$AA
	BEQ LBA89
	JMP LB55D

LBA89:	CPY #$55
	BEQ LBA90
	JMP LB55D

LBA90:	LDY #$01
	LDX $F7,Y
	CPX #$55
	BEQ LBA9B
	JMP LB55D

LBA9B:
	INY
	LDX $F7,Y
	CPX #$AA
	BEQ LBAA5
	JMP LB55D
LBAA5:
	LDY #$01
	LDX.W $0000,Y
	CPX #$55
	BEQ LBAB1
	JMP LB55D

LBAB1:
	DEY
	LDX.W $0000,Y
	CPX #$AA
	BEQ LBABC
	JMP LB55D

LBABC:
	LDX #$01
	LDY $F7,X
	CPY #$55
	BEQ LBAC7
	JMP LB55D
LBAC7:
	INX
	LDY $F7,X
	CPY #$AA
	BEQ LBAD1
	JMP LB55D

LBAD1:	LDX #$01
	LDY.W $0000,X
	CPY #$55
	BEQ LBADD
	JMP LB55D

LBADD:	DEX
	LDY.W $0000,X
	CPY #$AA
	BEQ LBAE8
	JMP LB55D

LBAE8:	JSR LB247
	; STX
	DB $FD,$00,$00,$33,$34,$38,$00,$00
	; STY
	DB $33,$34,$39,$FD

	LDX #$AA
	STX $F8
	STX.W $0001
	LDY #$55
	STY $F9
	STY.W $0000
	LDA $F8
	CMP #$AA
	BEQ LBB0E
	JMP LB55D

LBB0E:
	LDA $F9
	CMP #$55
	BEQ LBB17
	JMP LB55D

LBB17:
	LDA.W $0001
	CMP #$AA
	BEQ LBB21
	JMP LB55D
LBB21:
	LDA.W $0000
	CMP #$55
	BEQ LBB2B
	JMP LB55D
LBB2B:
	LDY #$01
	LDX #$55
	STX $F7,Y
	INY
	LDX #$AA
	STX $F7,Y
	LDA $F8
	CMP #$55
	BEQ LBB3F
	JMP LB55D
LBB3F:
	LDA $F9
	CMP #$AA
	BEQ LBB48
	JMP LB55D
LBB48:
	LDX #$00
	STX $F8
	STX.W $0001
	LDY #$FF
	STY $F9
	STY.W $0000
	LDA $F8
	BEQ LBB5D
	JMP LB55D
LBB5D:
	LDA $F9
	CMP #$FF
	BEQ LBB66
	JMP LB55D
LBB66:
	LDA.W $0001
	BEQ LBB6E
	JMP LB55D
LBB6E:
	LDA.W $0000
	CMP #$FF
	BEQ LBB78
	JMP LB55D
LBB78:
	LDX #$01
	TAY
	STY $F7,X
	INX
	LDY #$00
	STY $F7,X
	LDA $F8
	CMP #$FF
	BEQ LBB8B
	JMP LB55D
LBB8B:
	LDA $F9
	BEQ LBB92
	JMP LB55D

LBB92:	JSR LB247
	; INX
	DB $FD,$00,$00,$29,$2E,$38,$00,$00
	; DEX
	DB $24,$25,$38,$FD

	LDX #$7F
	BPL LBBA8
	JMP LB55D

LBBA8:
	INX
	BMI LBBAE
	JMP LB55D
LBBAE:
	LDX #$FF
	BNE LBBB5
	JMP LB55D
LBBB5:	
	INX
	BEQ LBBBB
	JMP LB55D
LBBBB:
	BPL LBBC0
	JMP LB55D
LBBC0:
	DEX
	BMI LBBC6
	JMP LB55D
LBBC6:
	BNE LBBCB
	JMP LB55D
LBBCB:
	LDX #$01
	DEX
	BEQ LBBD3
	JMP LB55D
LBBD3:
	JSR LB247
	; INY
	DB $FD,$00,$00,$29,$2E,$39,$00,$00
	; DEY
	DB $24,$25,$39,$FD
	LDY #$7F
	BPL LBBE9
	JMP LB55D

LBBE9:
	INY
	BMI LBBEF
	JMP LB55D
LBBEF:
	LDY #$FF
	INY
	BEQ LBBF7
	JMP LB55D
LBBF7:	
	BPL LBBFC
	JMP LB55D
LBBFC:
	DEY
	BMI LBC02
	JMP LB55D
LBC02:
	BNE LBC07
	JMP LB55D
LBC07:
	CPY #$FF
	BEQ LBC0E
	JMP LB55D
LBC0E:
	LDY #$01
	DEY
	BEQ LBC16
	JMP LB55D
LBC16:
	JSR LB247
	DB $FD,$00,$00,$21,$2E,$24,$00,$00
	DB $33,$22,$23,$FD
	LDA #$AA
	AND #$55
	BEQ LBC2E
	JMP LB55D
LBC2E:
	BPL LBC33
	JMP LB55D
LBC33:
	CMP #$00
	BEQ LBC3A
	JMP LB55D
LBC3A:
	LDA #$FF
	AND #$FF
	BMI LBC43
	JMP LB55D
LBC43:
	BNE LBC48
	JMP LB55D
LBC48:
	CMP #$FF
	BEQ LBC4F
	JMP LB55D
LBC4F:
	SEC
	LDA #$FF
	SBC #$FF
	BEQ LBC59
	JMP LB55D
LBC59:
	BPL LBC5E
	JMP LB55D
LBC5E:
	BVC LBC63
	JMP LB55D
LBC63:
	BCS LBC68
	JMP LB55D
LBC68:	LDA #$00
	BCS LBC6F
	JMP LB55D
LBC6F:
	LDA #$10
	ADC #$7F
	BMI LBC78
	JMP LB55D
LBC78:
	BCC LBC7D
	JMP LB55D
LBC7D:
	BVS LBC82
	JMP LB55D
LBC82:
	BNE LBC87
	JMP LB55D
LBC87:
	CMP #$90
	BEQ LBC8E
	JMP LB55D
LBC8E:
	JSR LB247
	; EOR
	DB $FD,$00,$00,$25,$2F,$32,$00,$00
	DB $2F,$32,$21,$FD

	LDA #$FF
	EOR #$FF
	BEQ LBCA6
	JMP LB55D
LBCA6:
	BPL LBCAB
	JMP LB55D
LBCAB:
	ORA #$FF
	BNE LBCB2
	JMP LB55D
LBCB2:
	BMI LBCB7
	JMP LB55D
LBCB7:
	CMP #$FF
	BEQ LBCBE
	JMP LB55D
LBCBE:
	LDA #$00
	ORA #$FF
	BMI LBCC7
	JMP LB55D
LBCC7:	
	BNE LBCCC
	JMP LB55D
LBCCC:
	CMP #$FF
	BEQ LBCD3
	JMP LB55D
LBCD3:
	ORA #$00
	CMP #$FF
	BEQ LBCDC
	JMP LB55D
LBCDC:
	JSR LB247
	; ASL
	DB $FD,$00,$00,$21,$33,$2C,$FD
	LDA #$40
	ASL A
	BCC LBCEE
	JMP LB55D
LBCEE:
	BNE LBCF3
	JMP LB55D
LBCF3:
	BMI LBCF8
	JMP LB55D
LBCF8:
	CMP #$80
	BEQ LBCFF
	JMP LB55D
LBCFF:
	ASL A
	BEQ LBD05
	JMP LB55D
LBD05:
	BPL LBD0A
	JMP LB55D
LBD0A:
	BCS LBD0F
	JMP LB55D
LBD0F:	LDA #$55
	STA $F9
	STA.W $0002
	ASL $F9
	LDA $F9
	CMP #$AA
	BEQ LBD1E
	; Original misses check here.
LBD1E:	LDX #$01 
LBD20:	ASL $F8,X
	LDA $F9
	CMP #$54
	BEQ LBD2B
LBD28:	JMP LB55D

LBD2B:	
	ASL.W $0001,X
	LDA.W $0002
	CMP #$AA
	BEQ LBD38
	JMP LB55D
LBD38:
	JSR LB247
	; ROL ROR
	DB $FD,$00,$00,$32,$2F,$2C,$00,$00,$32,$2F,$32
	; LSR
	DB $00,$00,$2C,$33,$32,$FD

	CLC
	LDA #$40
	ROL A
	BMI LBD55
	JMP LB55D
LBD55:
	BCC LBD5A
	JMP LB55D
LBD5A:
	BNE LBD5F
	JMP LB55D
LBD5F:
	CMP #$80
	BEQ LBD66
	JMP LB55D
LBD66:
	CLC
	ROL A
	BEQ LBD6D
	JMP LB55D
LBD6D:
	BCS LBD72
	JMP LB55D
LBD72:	
	BPL LBD77
	JMP LB55D
LBD77:
	SEC
	LDA #$40
	ROL A
	BMI LBD80
	JMP LB55D
LBD80:
	BCC LBD85
	JMP LB55D
LBD85:	
	BNE LBD8A
	JMP LB55D
LBD8A:
	CMP #$81
	BEQ LBD91
	JMP LB55D
LBD91:
	CLC
	ROL A
	BCS LBD98
	JMP LB55D
LBD98:
	BPL LBD9D
	JMP LB55D
LBD9D:
	CMP #$02
	BEQ LBDA4
	JMP LB55D
LBDA4:
	SEC
	LDA #$82
	LSR A
	BPL LBDAD
	JMP LB55D
LBDAD:
	BCC LBDB2
	JMP LB55D
LBDB2:
	BNE LBDB7
	JMP LB55D
LBDB7:
	CMP #$41
	BEQ LBDBE
	JMP LB55D
LBDBE:
	CLC
	LSR A
	BCS LBDC5
	JMP LB55D
LBDC5:
	CMP #$20
	BEQ LBDCC
	JMP LB55D
LBDCC:
	JSR LB247
	; CPX
	DB $FD,$00,$00,$23,$30,$38,$00,$00
	; CPY
	DB $23,$30,$39,$FD
	CLC
	LDX #$70
	CPX #$70
	BEQ LBDE5
	JMP LB55D
LBDE5:
	BCS LBDEA
	JMP LB55D
LBDEA:
	CPX #$71
	BMI LBDF1
	JMP LB55D
LBDF1:
	BCC LBDF6
	JMP LB55D
LBDF6:
	BNE LBDFB
	JMP LB55D
LBDFB: 
	CPX #$2C
	BCS LBE02
	JMP LB55D
LBE02:
	BNE LBE07
	JMP LB55D
LBE07:
	BPL LBE0C
	JMP LB55D
LBE0C:
	LDA #$55
	STA $F8
	STA.W $0001
	LDX #$55
	CPX $F8
	BEQ LBE1C
	JMP LB55D
LBE1C:
	CPX.W $0001
	BEQ LBE24
	JMP LB55D
LBE24:
	CPX #$55
	BEQ LBE2B
	JMP LB55D
LBE2B:
	CLC
	LDY #$30
	CPY #$30
	BEQ LBE35
	JMP LB55D
LBE35:
	BCS LBE3A
	JMP LB55D
LBE3A:
	CPY #$31
	BMI LBE41
	JMP LB55D
LBE41:
	BCC LBE46
	JMP LB55D
LBE46:
	BNE LBE4B
	JMP LB55D
LBE4B:
	CPY #$2C
	BCS LBE52
	JMP LB55D
LBE52:
	BNE LBE57
	JMP LB55D
LBE57:
	BPL LBE5C
	JMP LB55D
LBE5C:
	LDY #$55
	CPY $F8
	BEQ LBE65
	JMP LB55D
LBE65:
	CPY.W $0001
	BEQ LBE6D
	JMP LB55D
LBE6D:
	CPY #$55	
	BEQ LBE74
	JMP LB55D
LBE74:
	JSR LB247
	; CMP
	DB $FD,$00,$00,$23,$2D,$30,$FD
	CLC
	LDA #$30
	CMP #$30
	BEQ LBE88
	JMP LB55D
LBE88:
	BCS LBE8D
	JMP LB55D
LBE8D:
	CMP #$31
	BCC LBE94
	JMP LB55D
LBE94:
	BNE LBE99
	JMP LB55D
LBE99:
	BMI LBE9E
	JMP LB55D
LBE9E:
	CMP #$2C
	BPL LBEA5
	JMP LB55D
LBEA5:
	BNE LBEAA
	JMP LB55D
LBEAA:
	BCS LBEAF
	JMP LB55D
LBEAF:
	CMP #$30
	BEQ LBEB6
	JMP LB55D
LBEB6:
	LDA $F8
	CMP $F8
	BEQ LBEBF
	JMP LB55D
LBEBF:
	CMP.W $0001
	BEQ LBECA
	JMP LB55D
LBECA:
	LDA $F8,X
	CMP $F8,X
	BEQ LBED3
	JMP LB55D
LBED3:
	LDA.W $0000,X
	CMP.W $0000,X
	BEQ LBEDE
	JMP LB55D
LBEDE:
	LDA.W $0000,Y
	CMP.W $0000,Y
	BEQ LBEE9
	JMP LB55D
LBEE9:
	LDA ($F8,X)
	CMP ($F8,X)
	BEQ LBEF2
	JMP LB55D
LBEF2:
	LDA ($F8),Y
	CMP ($F8),Y
	BEQ LBEFB
	JMP LB55D
LBEFB:
	JMP LA000

	TIMES $BFFA-$ DB $FF
	
	DW LB000
	DW LB000
	DW LB000
	